"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseAriaSnapshot = parseAriaSnapshot;
var _utilsBundle = require("../utilsBundle");
var _utils = require("../utils");
/**
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function parseAriaSnapshot(text) {
  const fragment = _utilsBundle.yaml.parse(text);
  const result = {
    kind: 'role',
    role: 'fragment'
  };
  populateNode(result, fragment);
  return result;
}
function populateNode(node, container) {
  for (const object of container) {
    if (typeof object === 'string') {
      const childNode = parseKey(object);
      node.children = node.children || [];
      node.children.push(childNode);
      continue;
    }
    for (const key of Object.keys(object)) {
      node.children = node.children || [];
      const value = object[key];
      if (key === 'text') {
        node.children.push({
          kind: 'text',
          text: valueOrRegex(value)
        });
        continue;
      }
      const childNode = parseKey(key);
      if (childNode.kind === 'text') {
        node.children.push({
          kind: 'text',
          text: valueOrRegex(value)
        });
        continue;
      }
      if (typeof value === 'string') {
        node.children.push({
          ...childNode,
          children: [{
            kind: 'text',
            text: valueOrRegex(value)
          }]
        });
        continue;
      }
      node.children.push(childNode);
      populateNode(childNode, value);
    }
  }
}
function applyAttribute(node, key, value) {
  if (key === 'checked') {
    (0, _utils.assert)(value === 'true' || value === 'false' || value === 'mixed', 'Value of "disabled" attribute must be a boolean or "mixed"');
    node.checked = value === 'true' ? true : value === 'false' ? false : 'mixed';
    return;
  }
  if (key === 'disabled') {
    (0, _utils.assert)(value === 'true' || value === 'false', 'Value of "disabled" attribute must be a boolean');
    node.disabled = value === 'true';
    return;
  }
  if (key === 'expanded') {
    (0, _utils.assert)(value === 'true' || value === 'false', 'Value of "expanded" attribute must be a boolean');
    node.expanded = value === 'true';
    return;
  }
  if (key === 'level') {
    (0, _utils.assert)(!isNaN(Number(value)), 'Value of "level" attribute must be a number');
    node.level = Number(value);
    return;
  }
  if (key === 'pressed') {
    (0, _utils.assert)(value === 'true' || value === 'false' || value === 'mixed', 'Value of "pressed" attribute must be a boolean or "mixed"');
    node.pressed = value === 'true' ? true : value === 'false' ? false : 'mixed';
    return;
  }
  if (key === 'selected') {
    (0, _utils.assert)(value === 'true' || value === 'false', 'Value of "selected" attribute must be a boolean');
    node.selected = value === 'true';
    return;
  }
  throw new Error(`Unsupported attribute [${key}] `);
}
function parseKey(key) {
  const tokenRegex = /\s*([a-z]+|"(?:[^"]*)"|\/(?:[^\/]*)\/|\[.*?\])/g;
  let match;
  const tokens = [];
  while ((match = tokenRegex.exec(key)) !== null) tokens.push(match[1]);
  if (tokens.length === 0) throw new Error(`Invalid key ${key}`);
  const role = tokens[0];
  let name = '';
  let index = 1;
  if (tokens.length > 1 && (tokens[1].startsWith('"') || tokens[1].startsWith('/'))) {
    const nameToken = tokens[1];
    if (nameToken.startsWith('"')) {
      name = nameToken.slice(1, -1);
    } else {
      const pattern = nameToken.slice(1, -1);
      name = new RegExp(pattern);
    }
    index = 2;
  }
  const result = {
    kind: 'role',
    role,
    name
  };
  for (; index < tokens.length; index++) {
    const attrToken = tokens[index];
    if (attrToken.startsWith('[') && attrToken.endsWith(']')) {
      const attrContent = attrToken.slice(1, -1).trim();
      const [attrName, attrValue] = attrContent.split('=', 2);
      const value = attrValue !== undefined ? attrValue.trim() : 'true';
      applyAttribute(result, attrName, value);
    } else {
      throw new Error(`Invalid attribute token ${attrToken} in key ${key}`);
    }
  }
  return result;
}
function normalizeWhitespace(text) {
  return text.replace(/[\r\n\s\t]+/g, ' ').trim();
}
function valueOrRegex(value) {
  return value.startsWith('/') && value.endsWith('/') ? new RegExp(value.slice(1, -1)) : normalizeWhitespace(value);
}